/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package xpathprocessor;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

/**
 *
 * @author bar
 */
public class XPathProcessor {

    private Document doc;
    private DocumentBuilder docBuilder;
   
    /**
     * Constructor. Loads and validates the XML file. 
     * @param filename name of the file to load.
     */
    public  XPathProcessor(String filename) throws ParserConfigurationException, IOException, SAXException {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setValidating(true);
        docBuilder = dbf.newDocumentBuilder();
        doc = docBuilder.parse(filename);
    }
    
    /**
     * Displays the result in the form of a well-formed XML document.
     * @param result - list of nodes corresponding to the XPath expression.
     */
    public void show(NodeList result) throws ParserConfigurationException, TransformerException
    {
        Element root=doc.createElement("result");
        
        for(int i=0;i<result.getLength();i++)
        {
            if(result.item(i) instanceof Element)
                root.appendChild(result.item(i));
            else {
                Attr attr = (Attr)result.item(i);
                root.setAttribute(attr.getName(), attr.getTextContent());
            }
        }
        
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer trace = tf.newTransformer();
        trace.transform(new DOMSource(root), new StreamResult(System.out));
    }
    
    public NodeList evaluate(String xPathExpr) throws XPathExpressionException
    {
        XPathFactory xpf = XPathFactory.newInstance();
        XPath xPath = xpf.newXPath();
        return (NodeList) xPath.evaluate(xPathExpr, doc.getDocumentElement(), XPathConstants.NODESET);
    }
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            if(args.length != 2)
            {
                System.err.println("pouziti:\nXPathProcessor <xpath expression> <xml file>");
                System.exit(-1);
            }
            XPathProcessor xPathProc = new XPathProcessor(args[1]);
            xPathProc.show(xPathProc.evaluate(args[0]));
        } catch (TransformerException ex) {
            Logger.getLogger(XPathProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (XPathExpressionException ex) {
            Logger.getLogger(XPathProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(XPathProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(XPathProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SAXException ex) {
            Logger.getLogger(XPathProcessor.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private Document createDocument() throws ParserConfigurationException {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        docBuilder = dbf.newDocumentBuilder();
        return docBuilder.newDocument();
    }

    
}
